@@ -3,7 +3,7 @@
 /*
 htop - Action.h
 (C) 2015 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
@@ -20,25 +20,30 @@ in the source distribution for its full text.
 #include "Settings.h"
 #include "UsersTable.h"
 
+
 typedef enum {
-   HTOP_OK = 0x00,
-   HTOP_REFRESH = 0x01,
-   HTOP_RECALCULATE = 0x03, // implies HTOP_REFRESH
-   HTOP_SAVE_SETTINGS = 0x04,
-   HTOP_KEEP_FOLLOWING = 0x08,
-   HTOP_QUIT = 0x10,
-   HTOP_REDRAW_BAR = 0x20,
-   HTOP_UPDATE_PANELHDR = 0x41, // implies HTOP_REFRESH
+   HTOP_OK              = 0x00,
+   HTOP_REFRESH         = 0x01,
+   HTOP_RECALCULATE     = 0x02 | HTOP_REFRESH,
+   HTOP_SAVE_SETTINGS   = 0x04,
+   HTOP_KEEP_FOLLOWING  = 0x08,
+   HTOP_QUIT            = 0x10,
+   HTOP_REDRAW_BAR      = 0x20,
+   HTOP_UPDATE_PANELHDR = 0x40 | HTOP_REFRESH,
+   HTOP_RESIZE          = 0x80 | HTOP_REFRESH | HTOP_REDRAW_BAR | HTOP_UPDATE_PANELHDR,
 } Htop_Reaction;
 
+struct MainPanel_; // IWYU pragma: keep
+
 typedef struct State_ {
    Settings* settings;
    UsersTable* ut;
    ProcessList* pl;
-   Panel* panel;
+   struct MainPanel_* mainPanel;
    Header* header;
    bool pauseProcessUpdate;
    bool hideProcessSelection;
+   bool hideMeters;
 } State;
 
 static inline bool State_hideFunctionBar(const State* st) {
@@ -53,6 +58,8 @@ bool Action_setUserOnly(const char* userName, uid_t* userId);
 
 Htop_Reaction Action_setSortKey(Settings* settings, ProcessField sortKey);
 
+Htop_Reaction Action_setScreenTab(Settings* settings, int x);
+
 Htop_Reaction Action_follow(State* st);
 
 void Action_setBindings(Htop_Action* keys);
