@@ -3,7 +3,7 @@
 /*
 htop - DisplayOptionsPanel.h
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
@@ -11,6 +11,7 @@ in the source distribution for its full text.
 #include "ScreenManager.h"
 #include "Settings.h"
 
+
 typedef struct DisplayOptionsPanel_ {
    Panel super;
 
