@@ -7,7 +7,11 @@ Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include "config.h" // IWYU pragma: keep
+
+#ifndef HTOP_SOLARIS
 #include <signal.h>
+#endif
 
 #include "Panel.h"
 
