@@ -3,7 +3,7 @@
 /*
 htop - TraceScreen.h
 (C) 2005-2006 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
@@ -28,7 +28,7 @@ typedef struct TraceScreen_ {
 
 extern const InfoScreenClass TraceScreen_class;
 
-TraceScreen* TraceScreen_new(Process* process);
+TraceScreen* TraceScreen_new(const Process* process);
 
 void TraceScreen_delete(Object* cast);
 
