@@ -3,7 +3,7 @@
 /*
 htop - FreeBSDProcess.h
 (C) 2015 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
@@ -14,26 +14,13 @@ in the source distribution for its full text.
 #include "Settings.h"
 
 
-#define PROCESS_FLAG_FREEBSD_TTY   0x0100
-
-extern const char* const nodevStr;
-
 typedef struct FreeBSDProcess_ {
    Process super;
-   int   kernel;
    int   jid;
    char* jname;
-   const char* ttyPath;
+   char* emul;
 } FreeBSDProcess;
 
-static inline bool Process_isKernelThread(const Process* this) {
-   return ((const FreeBSDProcess*)this)->kernel == 1;
-}
-
-static inline bool Process_isUserlandThread(const Process* this) {
-   return this->pid != this->tgid;
-}
-
 extern const ProcessClass FreeBSDProcess_class;
 
 extern const ProcessFieldData Process_fields[LAST_PROCESSFIELD];
@@ -42,6 +29,4 @@ Process* FreeBSDProcess_new(const Settings* settings);
 
 void Process_delete(Object* cast);
 
-bool Process_isThread(const Process* this);
-
 #endif
