@@ -4,18 +4,15 @@
 htop - SolarisProcessList.h
 (C) 2014 Hisham H. Muhammad
 (C) 2017,2018 Guy M. Broome
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
-#define MAXCMDLINE 255
-
-#define GZONE "global    "
-#define UZONE "unknown   "
-
-#include "zfs/ZfsArcStats.h"
+#include "config.h" // IWYU pragma: keep
 
 #include <kstat.h>
+#include <stdbool.h>
+#include <stdint.h>
 #include <sys/param.h>
 #include <sys/uio.h>
 #include <sys/resource.h>
@@ -23,6 +20,15 @@ in the source distribution for its full text.
 #include <sys/sysinfo.h>
 #include <sys/swap.h>
 
+#include "Hashtable.h"
+#include "ProcessList.h"
+#include "UsersTable.h"
+
+#include "solaris/SolarisProcess.h"
+
+#include "zfs/ZfsArcStats.h"
+
+
 #define ZONE_ERRMSGLEN 1024
 extern char zone_errmsg[ZONE_ERRMSGLEN];
 
@@ -38,6 +44,7 @@ typedef struct CPUData_ {
    uint64_t lkrnl;
    uint64_t lintr;
    uint64_t lidle;
+   bool online;
 } CPUData;
 
 typedef struct SolarisProcessList_ {
@@ -47,20 +54,12 @@ typedef struct SolarisProcessList_ {
    ZfsArcStats zfs;
 } SolarisProcessList;
 
-char* SolarisProcessList_readZoneName(kstat_ctl_t* kd, SolarisProcess* sproc);
-
-ProcessList* ProcessList_new(UsersTable* usersTable, Hashtable* pidMatchList, uid_t userId);
+ProcessList* ProcessList_new(UsersTable* usersTable, Hashtable* dynamicMeters, Hashtable* dynamicColumns, Hashtable* pidMatchList, uid_t userId);
 
 void ProcessList_delete(ProcessList* pl);
 
-/* NOTE: the following is a callback function of type proc_walk_f
- *       and MUST conform to the appropriate definition in order
- *       to work.  See libproc(3LIB) on a Solaris or Illumos
- *       system for more info.
- */
-
-int SolarisProcessList_walkproc(psinfo_t* _psinfo, lwpsinfo_t* _lwpsinfo, void* listptr);
-
 void ProcessList_goThroughEntries(ProcessList* super, bool pauseProcessUpdate);
 
+bool ProcessList_isCPUonline(const ProcessList* super, unsigned int id);
+
 #endif
