@@ -90,7 +90,7 @@ void Platform_setBindings(Htop_Action* keys) {
    (void) keys;
 }
 
-int Platform_getUptime() {
+int Platform_getUptime(void) {
    return 0;
 }
 
@@ -100,7 +100,7 @@ void Platform_getLoadAverage(double* one, double* five, double* fifteen) {
    *fifteen = 0;
 }
 
-int Platform_getMaxPid() {
+int Platform_getMaxPid(void) {
    return 1;
 }
 
@@ -129,12 +129,6 @@ char* Platform_getProcessEnv(pid_t pid) {
    return NULL;
 }
 
-char* Platform_getInodeFilename(pid_t pid, ino_t inode) {
-   (void)pid;
-   (void)inode;
-   return NULL;
-}
-
 FileLocks_ProcessData* Platform_getProcessLocks(pid_t pid) {
    (void)pid;
    return NULL;
